/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.DialogInterface.OnDismissListener;
import android.content.Intent;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.PowerManager;
import android.provider.Settings;
import android.view.Window;
import android.view.WindowManager;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.tool.DataUtils;

import java.io.IOException;

/**     
  * 
  * @ProjectName:    
  * @Package:        net.micode.notes.ui
  * @ClassName:      AlarmAlertActivity
  * @Description:     AlarmAlertActivity类的作用是作为一个提醒警报的活动界面。
  * 当便签的提醒功能触发时，系统会启动AlarmAlertActivity，显示提醒的详细内容和相关操作选项，
  * 让用户可以对提醒进行处理，比如查看提醒内容、延迟提醒或者标记为已完成等操作。
  * 这个类主要负责提醒相关的界面展示和用户交互逻辑。
  * @Author:         xumingyang
  * @CreateDate:     2023-12-24 10:33
  * @UpdateUser:     更新者：
  * @UpdateDate:     2023-12-24 10:33
  * @UpdateRemark:   更新说明：
  * @Version:        1.0
 */
public class AlarmAlertActivity extends Activity implements OnClickListener, OnDismissListener {
    private long mNoteId;  //文本在数据库存储中的ID号
    private String mSnippet; //闹钟提示时出现的文本片段
    private static final int SNIPPET_PREW_MAX_LEN = 60;
    MediaPlayer mPlayer;


     /**
      * @method  onCreate
      * @description :
      * 初始化提醒界面的布局和逻辑。
      * 设置窗口特性，请求不显示标题栏，并在锁屏状态下显示提醒界面。
      * 获取传入该活动的Intent对象，并尝试从中获取便签的ID。
      * 使用获取到的便签ID获取便签的摘要信息，并进行相应的处理。
      * 创建一个MediaPlayer对象，用于播放提醒的声音。
      * 检查该便签是否在便签数据库中可见。
      * 如果便签可见，则显示操作对话框，并播放提醒音效。
      * 如果便签不可见，则关闭该界面。
      * @date: 2023-12-24 10:41
      * @author: xumingyang
      * @param
      * @return
      */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //Bundle类型的数据与Map类型的数据相似，都是以key-value的形式存储数据的
        //onsaveInstanceState方法是用来保存Activity的状态的
        //能从onCreate的参数savedInsanceState中获得状态数
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        //界面显示——无标题
        final Window win = getWindow();
        win.addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED);

        if (!isScreenOn()) {
            win.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON
                    //保持窗体点亮
                    | WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
                    //将窗体点亮
                    | WindowManager.LayoutParams.FLAG_ALLOW_LOCK_WHILE_SCREEN_ON
                    //允许窗体点亮时锁屏
                    | WindowManager.LayoutParams.FLAG_LAYOUT_INSET_DECOR);
        }//在手机锁屏后如果到了闹钟提示时间，点亮屏幕

        Intent intent = getIntent();

        try {
            mNoteId = Long.valueOf(intent.getData().getPathSegments().get(1));
            mSnippet = DataUtils.getSnippetById(this.getContentResolver(), mNoteId);
            //根据ID从数据库中获取标签的内容；
            //getContentResolver（）是实现数据共享，实例存储。
            mSnippet = mSnippet.length() > SNIPPET_PREW_MAX_LEN ? mSnippet.substring(0,
                    SNIPPET_PREW_MAX_LEN) + getResources().getString(R.string.notelist_string_info)
                    : mSnippet;
            //判断标签片段是否达到符合长度
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
            return;
        }

        mPlayer = new MediaPlayer();
        if (DataUtils.visibleInNoteDatabase(getContentResolver(), mNoteId, Notes.TYPE_NOTE)) {
            showActionDialog();
            //弹出对话框
            playAlarmSound();
            //闹钟提示音激发
        } else {
            finish();
            //完成闹钟动作
        }
    }

     /**
      * @method  isScreenOn
      * @description 判断屏幕是否锁屏，调用系统函数判断
      * @date: 2023-12-24 10:45
      * @author: xumingyang
      * @param 
      * @return bool 类型
      */
    private boolean isScreenOn() {
        PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
        return pm.isScreenOn();
    }

     /**
      * @method  playAlarmSound
      * @description:
      * 该段代码的作用是根据用户的设置，播放系统默认的闹钟铃声。
      * 并且可以支持在静音模式下播放闹钟铃声
      * @date: 2023-12-24 10:47
      * @author: xumingyang
      * @param 
      * @return 
      */
    private void playAlarmSound() {
        Uri url = RingtoneManager.getActualDefaultRingtoneUri(this, RingtoneManager.TYPE_ALARM);
        //调用系统的铃声管理URI，得到闹钟提示音
        int silentModeStreams = Settings.System.getInt(getContentResolver(),
                Settings.System.MODE_RINGER_STREAMS_AFFECTED, 0);

        if ((silentModeStreams & (1 << AudioManager.STREAM_ALARM)) != 0) {
            mPlayer.setAudioStreamType(silentModeStreams);
        } else {
            mPlayer.setAudioStreamType(AudioManager.STREAM_ALARM);
        }
        try {
            mPlayer.setDataSource(this, url);
            //方法：setDataSource(Context context, Uri uri)
            //解释：无返回值，设置多媒体数据来源【根据 Uri】
            mPlayer.prepare();
            //准备同步
            mPlayer.setLooping(true);
            //设置是否循环播放
            mPlayer.start();
            //开始播放
        } catch (IllegalArgumentException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            //e.printStackTrace()函数功能是抛出异常， 还将显示出更深的调用信息
            //System.out.println(e)，这个方法打印出异常，并且输出在哪里出现的异常
        } catch (SecurityException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IllegalStateException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }


     /**
      * @method  showActionDialog
      * @description
      * 创建一个AlertDialog.Builder对象，并设置对话框的标题为应用的名称（R.string.app_name）。
      * 将对话框的消息内容设置为mSnippet变量所存储的文本信息。
      * 设置对话框的积极按钮（Positive Button）的文本为“确定”（R.string.notealert_ok），并将当前类作为按钮的点击监听器。
      * 如果屏幕处于开启状态（通过isScreenOn()方法判断），则设置对话框的消极按钮（Negative Button）的文本为“进入”（R.string.notealert_enter），并将当前类作为按钮的点击监听器。
      * 调用dialog.show().setOnDismissListener(this)方法显示对话框，并设置当前类作为对话框消失时的监听器。
      * @date: 2023-12-24 10:51
      * @author: xumingyang
      * @param
      * @return
      */
    private void showActionDialog() {
        AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        /*
         * AlertDialog的构造方法全部是Protected的
         * 所以不能直接通过new一个AlertDialog来创建出一个AlertDialog。
         * 要创建一个AlertDialog，就要用到AlertDialog.Builder中的create()方法
         * 如这里的dialog就是新建了一个AlertDialog
        */
        dialog.setTitle(R.string.app_name);
        //为对话框设置标题
        dialog.setMessage(mSnippet);
        //为对话框设置内容
        dialog.setPositiveButton(R.string.notealert_ok, this);
        //给对话框添加"Yes"按钮
        if (isScreenOn()) {
            dialog.setNegativeButton(R.string.notealert_enter, this);
        }//对话框添加"No"按钮
        dialog.show().setOnDismissListener(this);
    }

     /**
      * @method  onClick
      * @description :
      * 根据点击的按钮类型执行相应的操作，
      * 如果点击的是对话框中的消极按钮，则打开一个新的便签编辑界面，并将当前笔记的id传递给该Activity。
      * @date: 2023-12-24 10:55
      * @author: xumingyang
      * @param 
      * @return 
      */
    public void onClick(DialogInterface dialog, int which) {
        switch (which) {
            //用which来选择click后下一步的操作
            case DialogInterface.BUTTON_NEGATIVE:
                //这是取消操作
                Intent intent = new Intent(this, NoteEditActivity.class);
                //实现两个类间的数据传输
                intent.setAction(Intent.ACTION_VIEW);
                //设置动作属性
                intent.putExtra(Intent.EXTRA_UID, mNoteId);
                //实现key-value对
                //EXTRA_UID为key；mNoteId为键
                startActivity(intent);
                //开始动作
                break;
            default:
                //这是确定操作'
                break;
        }
    }

     /**
      * @method  onDismiss
      * @description 在闹钟提醒对话框消失时，停止闹钟铃声并结束当前Activity的生命周期。
      * @date: 2023-12-24 10:57
      * @author: xumingyang
      * @param 
      * @return 
      */
    public void onDismiss(DialogInterface dialog) {
        stopAlarmSound();
        //停止闹钟声音
        finish();
        //完成该动作
    }
     /**
      * @method  stopAlarmSound
      * @description 停止正在播放的闹钟声音，并释放相关的资源。
      * @date: 2023-12-24 10:58
      * @author: xumingyang
      * @param 
      * @return 
      */
    private void stopAlarmSound() {
        if (mPlayer != null) {
            mPlayer.stop();
            //停止播放
            mPlayer.release();
            //释放MediaPlayer对象
            mPlayer = null;
        }
    }
}
