/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;

/**
  *
  * @ProjectName:
  * @Package:        net.micode.notes.ui
  * @ClassName:      AlarmInitReceiver
  * @Description:    AlarmInitReceiver类是小米便签应用中的一个广播接收器（BroadcastReceiver）
  * 其作用是在设备启动时接收系统发送的广播，并执行一些初始化操作
  * @Author:         xumingyang
  * @CreateDate:     2023-12-24 11:11
  * @UpdateUser:     更新者：
  * @UpdateDate:     2023-12-24 11:11
  * @UpdateRemark:   更新说明：
  * @Version:        1.0
 */


public class AlarmInitReceiver extends BroadcastReceiver {

    private static final String [] PROJECTION = new String [] {
        NoteColumns.ID,
        NoteColumns.ALERTED_DATE
    };
    //对数据库的操作，调用标签ID和闹钟时间
    private static final int COLUMN_ID                = 0;
    private static final int COLUMN_ALERTED_DATE      = 1;


     /**
      * @method  onReceive
      * @description 是在设备启动后，通过查询数据库找到需要提醒的标签，并设置对应的闹钟。
      * 这样，在设备启动完成后，小米便签应用可以重新加载标签的提醒功能，确保按时提醒用户。
      * @date: 2023-12-24 11:26
      * @author: xumingyang
      * @param *Cursor在这里的作用是通过查找数据库中的标签内容，找到和当前系统时间相等的标签
      * 从游标中获取提醒日期（alertDate）
      * 创建一个新的Intent对象（sender），并指定目标广播接收器（AlarmReceiver）
      * 创建PendingIntent：使用getBroadcast()方法创建一个用于启动广播的PendingIntent对象（pendingIntent）
      * @return 查询结果将以Cursor对象（c）返回
      */
    @Override
    public void onReceive(Context context, Intent intent) {
        long currentDate = System.currentTimeMillis();
        //System.currentTimeMillis()产生一个当前的毫秒
        //这个毫秒其实就是自1970年1月1日0时起的毫秒数
        Cursor c = context.getContentResolver().query(Notes.CONTENT_NOTE_URI,
                PROJECTION,
                NoteColumns.ALERTED_DATE + ">? AND " + NoteColumns.TYPE + "=" + Notes.TYPE_NOTE,
                new String[] { String.valueOf(currentDate) },
                //将long变量currentDate转化为字符串
                null);
        //Cursor在这里的作用是通过查找数据库中的标签内容，找到和当前系统时间相等的标签
        if (c != null) {
            if (c.moveToFirst()) {
                do {
                    long alertDate = c.getLong(COLUMN_ALERTED_DATE);
                    Intent sender = new Intent(context, AlarmReceiver.class);
                    sender.setData(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, c.getLong(COLUMN_ID)));
                    PendingIntent pendingIntent = PendingIntent.getBroadcast(context, 0, sender, 0);
                    AlarmManager alermManager = (AlarmManager) context
                            .getSystemService(Context.ALARM_SERVICE);
                    alermManager.set(AlarmManager.RTC_WAKEUP, alertDate, pendingIntent);
                } while (c.moveToNext());
            }
            c.close();
        }
    }
    /*
     * 然而通过网上查找资料发现，对于闹钟机制的启动，通常需要上面的几个步骤
     * 如新建Intent、PendingIntent以及AlarmManager等
     * 这里就是根据数据库里的闹钟时间创建一个闹钟机制
     */
}
